#!/usr/bin/env python

import subprocess
import os


def run_command(cmd):
	"""
	Run a shell command
	"""

	rv = subprocess.call(cmd, shell=True)
	return rv


def listPortNames():
	"""
	Returns the available port names in the project.

	Ports are directories inside the 'root/port/' directory (if available).
	Port names are the name of those directories.

	Returns:
	- list of port name strings (if available).
	- None if port dir is not available or if not any ports are available.
	"""
	rv = None
	portNameList = list()
	portPath = "port"
	if os.path.isdir(portPath):
		dirCandidateList = os.listdir(portPath)
		for dirCandidate in dirCandidateList:
			dirCandidatePath = os.path.join(portPath, dirCandidate)
			if os.path.isdir(dirCandidatePath):
				portNameList.append(dirCandidate)
		portNameList.sort()

	if len(portNameList) != 0:
		rv = portNameList

	return rv


def egrep(keywords, whole_words=False):
	"""
	Runs egrep

	param: keywords       Keywords to search for
	param: whole_words    Search for whole words
	"""
	egrep_flags = "-i -nr -R --color"

	if whole_words:
		egrep_flags += " -w"

	cmd = f"egrep {egrep_flags} '{keywords}' src/ inc/ port/ || true"
	rv = run_command(cmd)
	return rv



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Imports -----------------------------------------------------------------

import os
import sys
from datetime import date
# import sphinx_rtd_theme
# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('..'))


# -- Load information from module --------------------------------------------

about = {}
packageName = "macrame"
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, f"../{packageName}/__init__.py"), 'r', encoding='utf-8') as f:
	exec(f.read(), about)


# -- Project information -----------------------------------------------------

project = packageName
year = date.today().year
copyright = f"{year}, {about['__author__']}"
author = about['__author__']

# The full version, including alpha/beta/rc tags
version = about['__version__']
release = about['__version__']


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
	'sphinx_rtd_theme',
	'sphinx.ext.autodoc',
	'sphinx.ext.coverage',
	'sphinx.ext.napoleon',
	'sphinx.ext.viewcode'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = "sphinx_rtd_theme"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']



#!/usr/bin/env python

"""
Temporary test code
"""

import argparse
from buildutil.configParser import ConfigParser
from buildutil.configParser import array2Dict
from .core.cli import Command


class TestCommand(Command):
	"""
	Test Command
	"""

	def config(self):
		"""
		Configuration of arguments

		"""
		self.subparser.add_argument(
			'-f',
			'--file',
			help='A readable file',
			# metavar='FILE',
			type=argparse.FileType('r'),
			default=None)

	def run(self, args):
		"""
		Runs the command
		"""
		print(f"File: '{args.file}'")

		lst = array2Dict([
			# Section Key Default Options
			["MAKE", "PORT", "posix", {"posix", "stm32f072rb"}],
			["MAKE", "TARGET", "dbg", {"dbg", "rel"}],
			["MAKE", "test", "dbg", {"dbg", "rel"}],
		])

		ini_filepath = "setup.ini"
		parser = ConfigParser(ini_filepath, lst)
		parser.setenv()

		return 0



#!/usr/bin/env python

"""
Utility to build Assembly/C/C++ projects
"""

import sys

try:
	from pkg_resources import get_distribution, DistributionNotFound
except Exception as e:
	print(e)
	sys.exit(1)

try:
	__version__ = get_distribution(__name__).version
except DistributionNotFound:
	__version__ = "Unknown"

__author__ = "Kanelis Elias"
__email__ = "hkanelhs@yahoo.gr"
__license__ = "MIT"



#!/usr/bin/env python

"""
App for launching macrame
"""

import sys
from .commands import MyParser
from .commands import BuildCommand
from .commands import RunCommand
from .commands import CleanCommand
from .commands import InfoCommand
from .commands import TodoCommand
from .test import TestCommand


class App:
	"""
	Macrame application
	"""

	def __init__(self):
		"""
		Initialises the app
		"""

		self.parser = MyParser(
			"mac[rame]",
			"Utility to build Assembly/C/C++ projects",
			"Author: Kanelis Elias")
		BuildCommand("build", "builds the software")
		CleanCommand("clean", "remove the generated files")
		RunCommand("run", "executes the program")
		InfoCommand("info", "shows project specific information")
		TodoCommand("todo", "Lists programmer's todo/bug/fix keywords")
		TestCommand("test", "this is a test")

	def run(self):
		"""
		Runs the app
		"""
		return self.parser.handle()


def app_run():
	"""
	Convenient function to run the app
	"""

	app = App()
	rv = app.run()
	sys.exit(rv)


if __name__ == '__main__':
	app_run()



#!/usr/bin/env python3

import os
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))


# ------------------------------------------------------------------------------

def list_all_files_recursively(dirpath):
	filepath_list = list()
	for parent_path, _, filenames in os.walk(dirpath):
		for filename in filenames:
			filepath = os.path.join(parent_path, filename)
			filepath_list.append(filepath)

	return filepath_list


# ------------------------------------------------------------------------------

# Package name
packageName = "macrame"

# Load information needed by setup
about = {}
with open(os.path.join(here, f"{packageName}/__init__.py"), 'r', encoding='utf-8') as f:
	exec(f.read(), about)

# Long description
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
	long_description = f.read()

# Requirements
with open("requirements.txt") as f:
	dependencies = [line for line in f if "==" in line]
	dependencies = [s.rstrip() for s in dependencies]


# ------------------------------------------------------------------------------
# Setup config
setup(
	use_scm_version=True,
	setup_requires=['setuptools_scm'],

	name=packageName,
	packages=[packageName],
	version=about['__version__'],
	license=about['__license__'],
	description='Utility to build Assembly/C/C++ projects',
	long_description=long_description,
	long_description_content_type='text/markdown',
	author=about['__author__'],
	author_email=about['__email__'],
	url=f"https://github.com/TediCreations/{packageName}",
	download_url=f"https://github.com/TediCreations/{packageName}/archive/" + about['__version__'] + '.tar.gz',
	keywords=['build', 'make', 'util'],
	install_requires=dependencies,
	package_data={'macrame': ["../" + filepath for filepath in list_all_files_recursively('static/')]},
	include_package_data=True,
	entry_points={
		"console_scripts": [
			"mac = macrame.app:app_run",
			"macrame = macrame.app:app_run",
		]
	},
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Intended Audience :: Developers',
		'Topic :: Software Development :: Build Tools',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.6',
	],
)



#!/usr/bin/env python

from .exceptions import UserInputError
import argparse
# import argcomplete
import os
import sys

_commandList = []
_parser = None
_subparser = None


class Parser(object):

	"""
	Supports the creation of commandline argument subcommands
	with support from argparse.
	"""

	def __init__(self, name, description, epilog):
		"""
		Creates an argument parser

		name: The name of the program
		description: Text explaining what the program does
		epilog: Text at the end of the help section
		"""

		global _parser
		if _parser is not None:
			raise Exception("Could not init parser")
		_parser = argparse.ArgumentParser(
			prog=name,
			description=description,
			epilog=epilog,
			fromfile_prefix_chars='@')

		global _subparser
		_subparser = _parser.add_subparsers(dest='cmd', description="")

		self.parser = _parser
		self.config()
		_parser = self.parser

	def error(self, message):
		"""
		Print error message and exit
		"""
		# TODO: Is it good to quit from argparse like this?
		# _parser.error("\033[0;31m" + str(message) + "\033[0m")
		print("\033[0;31m[ERROR]\t" + str(message) + "\033[0m")
		sys.exit(1)

	def config(self):
		"""
		Configuration of arguments

		Example

		self.parser.add_argument(
			'-v', '--version',
			action='store_true',
			help="get the version of the system")
		"""
		pass

	def handle(self):
		# argcomplete.autocomplete(_parser)
		args = _parser.parse_args()
		subcommand = _parser.parse_args().cmd

		self.parser = _parser
		self.run(args)
		# _parser = self.parser

		rv = 0
		global _commandList
		for command in _commandList:
			cmd_name = command['name']
			cmd_callback = command['callback']
			if cmd_name == subcommand:
				try:
					rv = cmd_callback(args)
				except UserInputError as e:
					self.error(e)

		return rv

	def run(self, args):
		pass


class Command(object):
	"""
	Supports the creation of commandline argument subcommands
	with support from argparse.
	"""

	def __init__(self, name, help=None):
		"""
		Creates an argument command

		name: The name of the command
		help: Description of the command or None
		"""
		self.name = name
		global _parser
		if _parser is None:
			raise Exception(f"Could not append command '{self.name}' to the parser")

		global _subparser
		self.subparser = _subparser.add_parser(
			self.name,
			help=help)

		d = dict()
		d['name'] = self.name
		d['callback'] = self.run

		global _commandList
		_commandList.append(d)

		self.config()

	def error(self, message):
		"""
		Print error message and exit

		message: The error message
		"""
		# TODO: Is it good to quit from argparse like this?
		# _parser.error("\033[0;31m" + str(message) + "\033[0m")
		print("\033[0;31m[ERROR]\t" + str(message) + "\033[0m")
		sys.exit(1)

	def config(self):
		"""
		Configuration of arguments
		"""
		# Example
		# self.subparser.add_argument(
		# 	'-v', '--version',
		# 	action='store_true',
		# 	help=f"get the version of the {self.name} system")
		pass

	def run(self, args):
		"""
		Executes the command

		args: The command arguments
		"""
		print(f"Command '{self.name}' just run!")
		return 0

	def check_directory(self, directoryPath):
		"""
		Check if this is a valid directory
		else exit with a message.

		directoryPath: The directory path to check for validity
		"""
		if not os.path.isdir(directoryPath):
			self.error(f"The directory {directoryPath} does not exist")

	def getArgument(self, name):
		"""
		Gets the value of an argument

		name: The argument name
		"""

		args = vars(_parser.parse_args())
		rv = None
		try:
			rv = args[name]
		except KeyError:
			self.error(f"Argument '{name}' does not exist")

		return rv



#!/usr/bin/env python

"""
Parser and build commands
"""

import os
import sys
from .core.cli import Parser
from .core.cli import Command
from .core.utils import listPortNames
from .core.utils import egrep
from .makefile import BuildManager
from . import __version__


class MyParser(Parser):
	"""
	Parser for the buildsystem
	"""

	def config(self):
		"""
		Configuration of arguments
		"""

		cwd_path = os.path.abspath(os.getcwd())
		self.parser.add_argument(
			'-C', '--directory',
			default=cwd_path,
			help="changes current working directory")
		self.parser.add_argument(
			'-v', '--version',
			action='store_true',
			help="output version and exit")

	def run(self, args):
		"""
		Configuration of arguments
		"""
		# Version information
		if args.version:
			print(f"Version: {__version__}")
			sys.exit(0)

		# Working directory
		directory = os.path.abspath(args.directory)
		if not os.path.isdir(directory):
			self.error(f"The directory {directory} does not exist!")
		os.chdir(directory)


class BuildCommand(Command):
	"""
	Builds the software
	"""

	def config(self):
		"""
		Configuration of arguments
		"""

		# Local or remote makefile
		self.subparser.add_argument(
			'-r', '--remote',
			default=False,
			action='store_true',
			help="use the tools internal build system config files (excludes '--local')")
		self.subparser.add_argument(
			'-l', '--local',
			dest='remote',
			action='store_false',
			help="use the project's local build system config files (excludes '--remote')")

		# Port name
		self.subparser.add_argument(
			'-p', '--port',
			default="",
			type=str,
			help="the port name.")

	def run(self, args):
		"""
		Runs the command
		"""
		build_manager = BuildManager(
			port_name=args.port,
			use_local_makefile=not args.remote
		)
		rv = build_manager.build()

		return rv


class CleanCommand(Command):
	"""
	Removes generated files
	"""

	def config(self):
		"""
		Configuration of arguments
		"""

		# Local or remote makefile
		self.subparser.add_argument(
			'-r', '--remote',
			default=False,
			action='store_true',
			help="use the tools internal build system config files (excludes '--local')")
		self.subparser.add_argument(
			'-l', '--local',
			dest='remote',
			action='store_false',
			help="use the project's local build system config files (excludes '--remote')")

	def run(self, args):
		"""
		Runs the command
		"""
		build_manager = BuildManager(use_local_makefile=not args.remote)
		rv = build_manager.clean()

		return rv


class RunCommand(Command):
	"""
	Executes the program under development
	"""

	def config(self):
		"""
		Configuration of arguments
		"""

		# Local or remote makefile
		self.subparser.add_argument(
			'-r', '--remote',
			default=False,
			action='store_true',
			help="use the tools internal build system config files (excludes '--local')")
		self.subparser.add_argument(
			'-l', '--local',
			dest='remote',
			action='store_false',
			help="use the project's local build system config files (excludes '--remote')")

		# Port name
		self.subparser.add_argument(
			'-p', '--port',
			default="",
			type=str,
			help="the port name.")

	def run(self, args):
		"""
		Runs the command
		"""
		build_manager = BuildManager(
			port_name=args.port,
			use_local_makefile=not args.remote
		)
		rv = build_manager.run()

		return rv


class InfoCommand(Command):
	"""
	Shows project specific information
	"""

	def run(self, args):
		"""
		Runs the command
		"""
		cwd = self.getArgument("directory")
		project_name = os.path.basename(os.path.normpath(cwd))
		ports = listPortNames()

		txt = f"Project: {project_name}\n"
		txt += f"Ports:   {ports}\n"
		print(txt)

		return 0


class TodoCommand(Command):
	"""
	Lists programmer's todos
	"""

	def config(self):
		"""
		Configuration of arguments
		"""

		# Whole words
		self.subparser.add_argument(
			'-w', '--whole_words',
			action='store_true',
			help="search only for whole words")
		# Keywords
		self.subparser.add_argument(
			'-k', '--keywords',
			default=['todo', 'bug', 'fix'],
			action='store',
			type=str,
			nargs='*',
			help="keywords to search in the project")

	def run(self, args):
		"""
		Runs the command
		"""
		keywords = args.keywords
		whole_words = args.whole_words

		rv = 0
		if len(keywords) == 0:
			self.error("Please select one or more keywords")
		else:
			for keyword in keywords:
				rv = egrep(keyword, whole_words=whole_words)

		return rv
